#include "ScannerScore/ScannerSUser.h"

using namespace std;

//////////////////////////////////////////////////////////
////// Function contents to be defined by the user ///////
//////////////////////////////////////////////////////////
// WARNING: Don't change the prototypes of the template //
// functions  in this file since they are used by the   //
//     core routines. Only change their contents!       //
//////////////////////////////////////////////////////////


/////////////////////////////
// USER ANALYSIS TEMPLATES //
/////////////////////////////

//////////////////////////////////////////////////////////
/// Function for User defined initial calculations to be executed before the scan starts
void UserInitCalcs(void){

  //////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR INITIAL CALCULATIONS BEFORE THE SCAN STARTS //
  //////////////////////////////////////////////////////////////////////////

}

/////////////////////////////////////////////////////////////////////
//// User defined analysis function to be executed once for each valid parameter space point that was generated. This routine serves to accept or reject the point for example through testing experimental constraints.
bool UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing){
  //////////////////////////////////////////////////////
  // ENTER CODE FOR YOUR TESTS/OUTPUT DURING THE SCAN //
  //////////////////////////////////////////////////////

  ////////////////////////////////////////
  ////////// Final print out//////////////
  ////////////////////////////////////////


  cout<< "--- Masses ---"<< endl;
  for(size_t i=0; i!=Phi.size();++i){
    cout <<Mass[i]<< endl;
  }
  cout<< "--- Couplings ---"<< endl;
  for(size_t i=0; i!=L.size();++i){
    cout <<  L[i]<< endl;
  }
  cout<< "--- VEVs ---"<< endl;
  for(size_t i=0; i!=Phi.size();++i){
    cout <<  Phi[i]<< endl;
  }
  cout<<endl;

  return true;
}


//////////////////////////////////////////////////////////
/// Function for User defined final calculations to be executed after the scan is finished
void UserFinalCalcs(void){

  ////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR FINAL CALCULATIONS AFTER THE SCAN IS DONE //
  ////////////////////////////////////////////////////////////////////////

}

//////////////////////////////////////////
// USER DEFINED CORE TEMPLATE FUNCTIONS //
//////////////////////////////////////////
//These are functions which are called by the core code to preform tests of parametrise the scan (see descriptions below)

bool CheckStability(LambdaRef & L){
  //This example is for the 2HDM
  if(L[3]>0 && L[4]>0 && L[5]+sqrt(L[3]*L[4])>0 && L[5]+L[6]-abs(L[2])+sqrt(L[3]*L[4])>0)
    return true;
  else
    return false;
  
}

bool CheckGlobal(PhiRef & Phi,LambdaRef & L,Potential & V){
  
  ///////////////////////////////////////////
  ///// Your calculations here! /////////////
  ///////////////////////////////////////////

  //This example is for the 2HDM

  //// Compute discriminant D
  double kd = pow((L[3]/L[4]),0.25);
  double Disc=L[7]*(L[0]-kd*kd*L[1])*(Phi[6]/Phi[2]-kd); 
  if(Disc <= 0)    
    return false;//If condition not met, reject point 
   
  return true;
}

void MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi){
  //////////////////////////
  // Variables:
  //   PhiPar[] : Vector of VEV parameters
  //   Phi[] : Vector of VEVs
  ///////////////////////////
  // Description:
  //The purpose of this function is to provide for a parametrization of the VEVs such that they can be generated in more generic ways other than just in a hypercubic box. For example in the 2HDM model we may want to generate h_1=246*cos(beta) and h_2=246*sin(beta), i.e. on a circle, using beta as a parameter.
  //Here you must write down such a relation in the form Phi[i]=function(PhiPar[0],PhiPar[1],...). The parameter ranges have already been specified in the mathematica notebook.

  Phi[2]=246*cos(PhiPar[0]);
  Phi[6]=246*sin(PhiPar[0]);
}

void MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,vector< vector<double> > & MixInternal,RandGen & r){
  //////////////////////////
  // Variables:
  //   PhiPar[]          Vector of VEV parameters
  //   Phi[]             Vector of VEVs
  //   MixPar[]         Vector of Mixing matrix parameters
  //   MixInternal[i][j] Used defined mixing matrix for the mixing block. i runs over the mass eigenstates and j runs from 0 to Nmixing-1 (Nmixing is the number or mixing states) 
  //   r                 random number generator (called as r(mix,max)) in case the user whishes to change here the scan of MixPar[] more flexibly.
  ///////////////////////////
  // Description:
  //The purpose of this function is to provide for a parametrization of the mixing matrix when for simpler cases where an analytic parametrization is natural, just as for example in the 2HDM. Since at this point in the scan the VEVs have been generated (and possibly their parameters, if they have been re-parametrized), we assume a general structure where the mixing matrix parametrization can depend on such (constant) quantities (i.e. Phi, the VEVS and PhiPar, the parameters) and a new set of parameters (MixPar) whose ranges have been specified in the Mathematica notebook. 
  //Contrarily to the VEV re-parametrization, we allow here the use the option to actually change the values of MixPar (i.e. override the values that have been generated in the ranges defined in the mathematica notebook) so that relations between Mixing parameters and VEV parameters are possible (such as beta=alpha-Pi/2 in the decoupling limit of the 2HDM).
  //Here you must write down: 
  //  i) (this one is optional) a relation MixPar[i]=function(Phi[0],...,PhiPars[0],...)
  MixPar[0]=PhiPar[0]-acos(-1)/2e0;
  
  //  ii) Expressions for the mixing matrix elements in the form MixInternal[i][j]=Function(MixPar[0],...,Phi[0],...,PhiPars[0],...), where the index i runs over the scalar eigenstates and the index j runs over the original mixing scalar states according to the order defined in the mathematica file. IMPORTANT NOTE: You are advised to check (in VERBOSE mode) that the ordering is indeed as you think it is. Note for example that the mass eigenstates are only ordered in mass if you enforce this in the Mathematica file! The work around for example in the 2HDM mode is to split the scan in 2 scans: The first, where the SM Higgs is the Heavy (and this state mass[0] fixed to 126) and the other is the ligth one (mass[1] < 126) and then another scan where the SM Higgs is the light (and this state mass[1] fixed at 126) and the other is the Heavy one (mass[0] > 126)

  MixInternal[0][0]=cos(MixPar[0]);
  MixInternal[0][1]=-sin(MixPar[0]);
  MixInternal[1][0]=sin(MixPar[0]);
  MixInternal[1][1]=cos(MixPar[0]);
}

void MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const vector< vector<double> > & Mixing,const MixingparamVec & MixPar,vector<double> & L,vector<double> & Mass, MLparamVec & MLPar, vector<double> & IndependentMs,vector<double> & IndependentLs,RandGen & r){
  //////////////////////////
  // Variables:
  //   PhiPar[]        Vector of VEV parameters
  //   Phi[]           Vector of VEVs
  //   MixPar[]       Vector of Mixing matrix parameters
  //   Mixing[i][j]    Full mixing matrix (diagonal directions also present in the lines below the mixing directions) i runs over all mass eigenstates and j runs over all initial (gauge) scalar states according to the Mathematica ordering 
  //   r               random number generator (called as r(mix,max)) in case the user whishes to change here the scan of MixPar[] more flexibly.
  ///////////////////////////
  // Description:
  //The purpose of this function is to provide a way of imposing extra relations in the scan among the last masses and couplings that are left to be generated at the end, through a parametrization, and/or with all the other quantities that have already been generated in in the earlier stages of the scan step. 
  //IMPORTANT NOTE: it is crucial in this case, that you check in Verbose mode that the relations you impose below are among  independent masses and couplings (in VERBOSE mode there are some lines at the end identifying the last independent quantities that are generated). If not, you may have to re-order your fields and couplings in the input mathematica notebook (the rule is that the prigram tries to leave for last, the last masses and couplings in the numbering adopted in the Mathematica notebook). In any case, below you will have to "declare" which couplings and masses you want to constrain so the code will crash and issue a warning if what you are trying to do is not allowed.
  // Since at this point in the scan the VEVs, and Mixings have been generated (and possibly their parameters, if they have been re-parametrized), we assume a general structure where the relations you impose may depend on all such (constant) quantities  and a new set of parameters (MLPar) whose ranges have been specified in the Mathematica notebook. 
  //Again, we allow here the use the option to actually change the values of MLPar (i.e. override the values that have been generated in the ranges defined in the mathematica notebook).

  //Here you must write down: 
  //  i) (WARNING: this one is NOT optional) specify here which masses and couplings (in the allowed set of independent couplings) you are going to use in your parametrization
  IndependentMs.push_back(1); //Mass[1] here for example will re-parametrised
  IndependentMs.push_back(2); //and Mass[2] as well
  //and couplings (in this example we comment the next line and use only relations among masses)
  ////IndependentLs.push_back(7);  

  // ii) Now write here your relations (in this example, this re-parametrization sets two masses to be equal)
  Mass[1]=MLPar[0];
  Mass[2]=MLPar[0];

}





