#ifndef SCANNERSMODELCLASSES_H
#define SCANNERSMODELCLASSES_H
#include "ScannerSTools.h"
#include <cmath>

class PhiVec;

typedef PhiVec PhiParamVec;
typedef VecTriplet MLparamVec;
typedef VecTriplet MixingparamVec;

class PhiVec: public VecTriplet {
 public:
  friend class Monomial;
  void generate(); //function to generate the vev space uniformly
  void generate(double (*)(const std::vector<double> &)); // version with a function defining a pdf to re-weigh or reject regions 
  //void Parametrization(const PhiParamVec & PhiPar,void (*PhiParametrization)(const PhiParamVec &,PhiVec &));//Use parametrization instead
  void setphi(const std::vector<double> & phi_in); // Function to rese the values of phi
  PhiVec(RandGen & r_in):r(r_in){}//Consructor must include a reference to the random number generator
 private:
  RandGen & r; // reference to random number generator in use 
};

class Couplings{
 public:
  typedef VecTriplet::size_type size_type;
  friend std::istream & operator>>(std::istream & in, Couplings & C){in>>C.L>>C.Mass; return in;} //Overloaded input operator to initialise
  friend std::ostream & operator<<(std::ostream & os, const Couplings & C){os<<C.L<<"\t"<<C.Mass; return os;} //Overloaded input operator to initialise
  Couplings(RandGen & r_in):r(r_in){}//Consructor must include a reference to the random number generator
  size_type size()const{return L.size();}
  double & operator[](size_type i){return L[i];}
  double operator[](size_type i)const{return L[i];}
  VecTriplet L; // Triplet to hold lagrangian couplings and ranges
  std::vector<unsigned int> Lindp;
  VecTriplet Mass; //Triplet to hold masses squared and ranges
  std::vector<unsigned int> Mindp;
 private:
  RandGen & r; // reference to random number generator in use
};

class deriv2V{
 public:
  typedef std::vector<double>::size_type size_type;
  typedef std::vector< std::vector< std::vector<double> > >::iterator iterator;
  std::vector< std::vector<double> > & operator[](size_type i){return d2V[i];}
  deriv2V(){}
  deriv2V(const std::vector< std::vector< std::vector<double> > > & Vec):d2V(Vec),Mag(Vec.size()){}
  double mag(size_type i)const{return Mag[i];}
  iterator begin(){return d2V.begin();}
  iterator end(){return d2V.end();} 
  size_type size(){return d2V.size();}
  std::vector<double> Mag;
 private:
  std::vector< std::vector< std::vector<double> > > d2V;
};

class Potential{
  friend class PhiRef;
  friend class LambdaRef;
  friend class MassRef;
  friend class MmixingRef;
  friend void PrintHeader(class PhiRef & Phi,class LambdaRef & L,class MassRef & Mass,Potential & V,unsigned int seed);
 public:
  typedef VecTriplet::size_type size_type;
  Potential(RandGen & r_in):r(r_in),Phi(r_in),PhiPar(r_in),Cp(r_in){}//Constructor must include a reference to the random number generator
  friend std::istream & operator>>(std::istream & in, Potential & V);
  friend std::ostream & operator<<(std::ostream & os, const Potential & V);
  double operator()() const;
  double operator()(size_type a) const;
  double operator()(std::vector< std::vector<Monomial> >::const_iterator aiter) const;
  double operator()(size_type i, size_type a) const;
  double operator()(size_type i, size_type j, size_type a) const;
  bool generate();
  void a0treeIni();
  bool FindMixing();
  void printdV();
  void printd2V();
  void printa0tree();
  bool CheckTreeUni();
  bool CheckBounds();
  void BoundTreeUni();
  void PrintHeader();
  
 private:
  RandGen & r; // reference to random number generator in use   
  PhiVec Phi;
  PhiParamVec PhiPar;
  MLparamVec MLpars;
  MixingparamVec MixingPars;
  Couplings Cp;
  std::vector<int> degeneracies;
  std::vector< std::vector<Monomial> > Mon;
  std::vector< std::vector<double> > dV;
  std::vector< std::vector<double> > d2Vreduced;
  unsigned int NIC,NIC2,Nmixed,NnoMixCurv,NGold,Nblocks;
  std::vector<int> BlockInfo; // NEW
  std::vector<unsigned int> BlockDims; //NEW
  std::vector<unsigned int> BlockPermutLabels; //NEW
  std::vector<unsigned int> orderdV;
  std::vector<unsigned int> orderd2V;
  deriv2V d2V,d2Vnew;
  //std::vector< std::vector< std::vector< std::vector<double> > > > d2V;
  std::vector< std::vector< std::vector<double> > > a0tree;
  double * a0treeSum;
  double * TempM;
  double * TempM2;
  double * TempOut;
  std::vector< std::vector<double> > Mbasis,Maux,Mrot,Mrot2,Mmixing;
};

class PhiRef{
 public:
 PhiRef(Potential & Vin):Phi_(Vin.Phi){}
  void copy(PhiRef & PhiIn){for(size_t i=0;i!=PhiIn.size();++i){(*this)[i]=PhiIn[i];} }
  double & operator[](size_t i){return Phi_[i];}
  size_t size()const{return Phi_.size();}
  friend std::ostream & operator<<(std::ostream & os, const PhiRef & P);
  PhiVec & Phi_;
};

class LambdaRef{
 public:
  LambdaRef(Potential & Vin):L_(Vin.Cp.L){}
  void copy(LambdaRef & LIn){for(size_t i=0;i!=LIn.size();++i){(*this)[i]=LIn[i];} }
  double & operator[](size_t i){return L_[i];}
  size_t size()const{return L_.size();}
  friend std::ostream & operator<<(std::ostream & os, const LambdaRef & L); 
  VecTriplet & L_;
};

class MassRef{
 public:
  MassRef(Potential & Vin):M_(Vin.Cp.Mass){}
  double operator[](size_t i){return M_[i];}
  size_t size()const{return M_.size();}
  const VecTriplet & M_;
};

class MmixingRef{
 public:
  MmixingRef(Potential & Vin):Mmix_(Vin.Mmixing),Nmixed_(Vin.Nmixed),NnoMixCurv_(Vin.NnoMixCurv),NGold_(Vin.NGold){}
  const std::vector<double> & operator[](size_t i){return Mmix_[i];}
  size_t size()const{return Mmix_.size();}
  size_t Nmixed(){return Nmixed_;}
  size_t NDiagMassive(){return NnoMixCurv_;}
  size_t Nmassless(){return NGold_;}
 private:
  const std::vector< std::vector<double> > & Mmix_;
  unsigned int & Nmixed_,NnoMixCurv_,NGold_;
};


void MyPhiParametrization(const PhiParamVec &,PhiVec &);
void MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi, MixingparamVec & MixingPars,std::vector< std::vector<double> > & MixInternal,RandGen & r);
void MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const std::vector< std::vector<double> > & Mixing,const MixingparamVec & MixPars,std::vector<double> & L,std::vector<double> & Mass, MLparamVec & MLparams, std::vector<double> & InDependentMs,std::vector<double> & InDependentLs,RandGen & r);

#endif 
